class PlaylistModes(object):
    """
    Class that contains all modes to create a playlist by.
    """
    SONGS = 0  # By songs
    ARTISTS = 1  # By artists
    GENRES = 2  # By genres
    ALBUMS = 3  # By albums
